#!/usr/bin/perl
# 
# Script to make bitmap image from binary data.
# 
# Usage:
# bin2xpm.pl [-width <width>] [-offset <offset>] [-size <size>] \
#            [-separator <separator>]
# 
# Example:
# cat sample.zip | ./bin2xpm.pl > sample.xpm
# 
# <width>      width of bitmap image. (0: auto select)
#              x    ... 2 times of the auto selected value.
#              /    ... 1/2 of the auto selected value.
#              xN   ... N times of the auto selected value.
#              /N   ... 1/N of the auto selected value.
#              xM/N ... M/N of the auto selected value.
#              xx   ... 4 times of the auto selected value.
# <offset>     offset to skip data.
# <size>       size to make bitmap image. (0: auto select)
# <separator>  size of separator. (HEX)

while ($_ = shift(@ARGV)) {
	if    ($_ eq "-width"    ) { $width     =     shift(@ARGV);  }
	elsif ($_ eq "-offset"   ) { $offset    =     shift(@ARGV);  }
	elsif ($_ eq "-size"     ) { $size      =     shift(@ARGV);  }
	elsif ($_ eq "-separator") { $separator = hex(shift(@ARGV)); }
}

$i = $datasize = 0;
while (defined($c = getc())) {
	if (($size > 0) && ($i > $offset + $size)) {
		last;
	}
	if ($i >= $offset) {
		$b[$datasize++] = $c;
	}
	$i++;
}

if ($size == 0) {
	$size = $datasize;
}

$width_rate = 1;
while ($width =~ /^[x\/]/) {
	($_, $r, $width) = $width =~ /^([x\/])(\d*)(.*)/;
	if ($r == 0) {
		$r = 2;
	}
	if ($_ eq "x") {
		$width_rate *= $r;
	} else {
		$width_rate /= $r;
	}
}

if ($width == 0) {
	for ($i = 1;; $i *= 2) {
		if ($size <= $i * $i * 8) {
			$width = $i;
			last;
		}
	}
	$width = int($width * $width_rate);
}

if ($separator > 0) {
	$separator = int($separator / $width) * $width;
}

$height = int(($size + $width - 1) / $width);

for ($i = 0; $i < 0x100; $i++) {
	$c = pack("C", $i);
	if ($c eq "\0") { # null
		$d = "black";
#	} elsif (($c eq "\r") || ($c eq "\n")) { # newline
#		$d = "blue";
#	} elsif ($c eq "\t") { # tab
#		$d = "lightblue";
	} elsif ($c eq "\xff") { # 0xff
		$d = "white";
	} elsif ($i < 0x20) { # control code
		$d = sprintf "#00%02x00", $i * 4 + 0x80; # RGB
	} elsif ($i < 0x80) { # text
		$d = sprintf "#%02x0000", $i + 0x80; # RGB
	} else { # binary
		$d = sprintf "#00%02x%02x", $i, $i; # RGB
	}
	$color[$i] = $d;
}

$colors = "";
for ($i = 0; $i < 0x100; $i++) {
	$n = sprintf "%02x", $i;
	$colors .= sprintf "\"%s c %s\",\n", $n, $color[$i];
}

$bitmap = "";
$i = $h = 0;

for ($y = 0; $y < $height; $y++) {
	$bitmap .= "\"";
	for ($x = 0; $x < $width; $x++) {
		if (($i < $size) && ($i < $datasize)) {
			$c = $b[$i];
		} else {
			$c = "\0";
		}
		$i++;
		$d = unpack("H2", $c);
		$bitmap .= "$d";
	}
	$bitmap .= "\",\n";
	$h++;

	if (($separator > 0) && (($i % $separator) == 0)) {
		$bitmap .= "\"";
		for ($x = 0; $x < $width; $x++) {
			$bitmap .= "ff";
		}
		$bitmap .= "\",\n";
		$h++;
	}
}

$header = "";
$header .= "/* XPM */\n";
$header .= "static char *bitmap_xpm[] = {\n";
$header .= "/* width height num_colors chars_per_pixel */\n";
$header .= "\"$width $h 256 2\",\n";

$tailer = "};\n";

print "$header";
print "$colors";
print "$bitmap";
print "$tailer";
