#!/usr/bin/perl
# 
# Script to dump binary data.
# 
# Usage:
# bindump.pl [-offset <offset>] [-size <size>] [-width <width>] \
#            [-height <height>] [-separator <sep>] [-ascii-separator <asep>] \
#            [-addr-format <afmt>] [-format <fmt>] [-hide <positions>]
# 
# Example:
# cat sample.zip | ./bindump.pl
# 
# <offset>  offset to skip data.
# <size>    size to output. (0: auto select)
# <width>   width of output. (0: auto select)
# <height>  height of output. (0: auto select)
# <sep>     size of separator of data field. (0: auto select, 1: none)
# <asep>    size of separator of ascii field. (0: auto select, 1: none)
# <afmt>    print format of address.
# <fmt>     print format of data.
# <positions> comma separated positions to hide. (Ex: -2,4,6-8,0xA-0xF,20-)

while ($_ = shift(@ARGV)) {
	if    ($_ eq "-offset"         ) { $offset = shift(@ARGV); }
	elsif ($_ eq "-size"           ) { $size   = shift(@ARGV); }
	elsif ($_ eq "-width"          ) { $width  = shift(@ARGV); }
	elsif ($_ eq "-height"         ) { $height = shift(@ARGV); }
	elsif ($_ eq "-separator"      ) { $sep    = shift(@ARGV); }
	elsif ($_ eq "-ascii-separator") { $asep   = shift(@ARGV); }
	elsif ($_ eq "-addr-format"    ) { $afmt   = shift(@ARGV); }
	elsif ($_ eq "-format"         ) { $fmt    = shift(@ARGV); }
	elsif ($_ eq "-hide"           ) { $hide   = shift(@ARGV); }
}

$i = $datasize = 0;
while (defined($c = getc())) {
	if (($size > 0) && ($i > $offset + $size)) {
		last;
	}
	if ($i >= $offset) {
		$b[$datasize++] = $c;
	}
	$i++;
}

if ($size == 0) {
	$size = $datasize;
}

if ($width == 0) {
	$width = 16;
}

if ($sep == 0) {
	$sep = 4;
}

if ($asep == 0) {
	$asep = 8;
}

if ($afmt eq "") {
	$afmt = "%04X";
}

if ($fmt eq "") {
	$fmt = "%02X";
}

if ($hide ne "") {
	@hides = split(/,/, $hide);
}

$i = 0;
for ($y = 0;; $y++) {
	if ($height > 0) {
		if ($y >= $height) {
			last;
		}
	} else {
		if ($i >= $size) {
			last;
		}
	}

	$addr = sprintf($afmt, $offset + $i);
	$rdump = "";
	$hdump = "";
	$ascii = "";
	for ($x = 0; $x < $width; $x++) {
		if ($x > 0) {
			if (($sep > 1) && (($x % $sep) == 0)) {
				$rdump .= " ";
				$hdump .= " ";
			}
			if (($asep > 1) && (($x % $asep) == 0)) {
				$ascii .= " ";
			}
			$rdump .= " ";
			$hdump .= " ";
		}
		if (($i < $size) && ($i < $datasize)) {
			$c = $b[$i];
			$d = unpack("C", $c);
			if (($d >= 0x20) && ($d <= 0x7e)) {
				$a = $c;
			} else {
				$a = ".";
			}
			$d = sprintf($fmt, $d);
			$rdump .= $d;
			foreach (@hides) {
				if ($_ eq "") {
					next;
				}
				if (/\-/) {
					($min, $max) = split(/\-/);
					if ($min eq "") {
						$min = -1;
					}
					if ($max eq "") {
						$max = -1;
					}
				} else {
					$min = $_;
					$max = $_;
				}
				if ($min =~ /^0x/) {
					$min = hex($min);
				}
				if ($max =~ /^0x/) {
					$max = hex($max);
				}
				if ((($min < 0) || ($offset + $i >= $min)) &&
				    (($max < 0) || ($offset + $i <= $max))) {
					$d =~ s/./X/g;
					$a =~ s/./X/g;
					last;
				}
			}
			$hdump .= $d;
			$ascii .= $a;
		} else {
			$rdump .= "  ";
			$hdump .= "  ";
			$ascii .= " ";
		}
		$i++;
	}

	if (($rdump =~ /\S/) && ($rdump eq $old)) {
		$same++;
	} else {
		$same = 0;
		$old = $rdump;
	}

	if ($same < 2) {
		print "$addr: $hdump  $ascii\n";
	} elsif ($same == 2) {
		print "... Skipped same lines ...\n";
	} else {
		redo;
	}
}
